@@ -1,6 +1,6 @@
 ;;; consult-register.el --- Consult commands for registers -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021-2023 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2024 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -67,8 +67,7 @@ Each element of the list must have the form (char . name).")
   "Describe marker register VAL."
   (with-current-buffer (marker-buffer val)
     (save-excursion
-      (save-restriction
-        (widen)
+      (without-restriction
         (goto-char val)
         (let* ((line (line-number-at-pos))
                (str (propertize (consult--line-with-mark val)
@@ -154,8 +153,7 @@ If COMPLETION is non-nil format the register for completion."
                (`(,str . ,props) (consult-register--describe val)))
     (when (string-search "\n" str)
       (let* ((lines (seq-take (seq-remove #'string-blank-p (split-string str "\n")) 3))
-             (space (apply #'min most-positive-fixnum
-                           (mapcar (lambda (x) (string-match-p "[^ ]" x)) lines))))
+             (space (cl-loop for x in lines minimize (string-match-p "[^ ]" x))))
         (setq str (mapconcat (lambda (x) (substring x space))
                              lines (concat "\n" (make-string (1+ key-len) ?\s))))))
     (setq str (concat
@@ -172,11 +170,14 @@ If COMPLETION is non-nil format the register for completion."
 (defun consult-register--alist (&optional noerror filter)
   "Return register list, sorted and filtered with FILTER.
 Raise an error if the list is empty and NOERROR is nil."
-  (or (sort (seq-filter
-             ;; Sometimes, registers are made without a `cdr'.
-             ;; Such registers don't do anything, and can be ignored.
-             (lambda (x) (and (cdr x) (or (not filter) (funcall filter x))))
-             register-alist)
+  (or (sort (cl-loop for reg in register-alist
+                     ;; Sometimes, registers are made without a `cdr' or with
+                     ;; invalid markers.  Such registers don't do anything, and
+                     ;; can be ignored.
+                     if (and (cdr reg)
+                             (or (not (markerp (cdr reg))) (marker-buffer (cdr reg)))
+                             (or (not filter) (funcall filter reg)))
+                     collect reg)
             #'car-less-than-car)
       (and (not noerror) (user-error "All registers are empty"))))
 
@@ -283,7 +284,7 @@ This function is derived from `register-read-with-preview'."
             (setq action (logxor #x8000000 key)))
            ((characterp key)
             (setq reg key))
-           (t (error "Non-character input"))))
+           (t (user-error "Non-character input"))))
       (when (timerp timer)
         (cancel-timer timer))
       (let ((w (get-buffer-window buffer)))
