@@ -88,31 +88,28 @@ See `xref-show-xrefs-function' for the description of the
 FETCHER and ALIST arguments."
   (let* ((consult-xref--fetcher fetcher)
          (candidates (consult-xref--candidates))
-         (display (alist-get 'display-action alist)))
+         (display (alist-get 'display-action alist))
+         (this-command #'consult-xref))
     (unless candidates
       (user-error "No xref locations"))
     (xref-pop-to-location
      (if (cdr candidates)
-         (apply
-          #'consult--read
+         (consult--read
           candidates
-          (append
-           (consult--customize-get #'consult-xref)
-           (list
-            :prompt "Go to xref: "
-            :history 'consult-xref--history
-            :require-match t
-            :sort nil
-            :category 'consult-xref
-            :group #'consult--prefix-group
-            :state
-            ;; do not preview other frame
-            (when-let (fun (pcase-exhaustive display
-                             ('frame nil)
-                             ('window #'switch-to-buffer-other-window)
-                             ('nil #'switch-to-buffer)))
-              (consult-xref--preview fun))
-            :lookup (apply-partially #'consult--lookup-prop 'consult-xref))))
+          :prompt "Go to xref: "
+          :history 'consult-xref--history
+          :require-match t
+          :sort nil
+          :category 'consult-xref
+          :group #'consult--prefix-group
+          :state
+          ;; do not preview other frame
+          (when-let (fun (pcase-exhaustive display
+                           ('frame nil)
+                           ('window #'switch-to-buffer-other-window)
+                           ('nil #'switch-to-buffer)))
+            (consult-xref--preview fun))
+          :lookup (apply-partially #'consult--lookup-prop 'consult-xref))
        (get-text-property 0 'consult-xref (car candidates)))
      display)))
 
